/*
 * @author mchyzer $Id: GrouperWsRestGetSubject.java,v 1.1 2008/03/29 10:50:43 mchyzer Exp $
 */
package edu.internet2.middleware.grouper.ws.rest.method;

import java.util.List;

import edu.internet2.middleware.grouper.util.GrouperUtil;
import edu.internet2.middleware.grouper.ws.GrouperWsVersion;
import edu.internet2.middleware.grouper.ws.rest.GrouperRestInvalidRequest;
import edu.internet2.middleware.grouper.ws.rest.GrouperServiceRest;
import edu.internet2.middleware.grouper.ws.rest.WsRequestBean;
import edu.internet2.middleware.grouper.ws.rest.WsResponseBean;
import edu.internet2.middleware.grouper.ws.rest.group.WsRestGetGroupsLiteRequest;
import edu.internet2.middleware.grouper.ws.util.GrouperServiceUtils;

/**
 * all first level resources on a get request
 */
public enum GrouperWsRestGetSubject {

  /** group get requests for groups */
  groups {

    /**
     * handle the incoming request based on HTTP method GET and group as resource, and members as subresource
     * @param clientVersion version of client, e.g. v1_3_000
     * @param urlStrings not including the app name or servlet.  
     * for http://localhost/grouper-ws/servicesRest/xhtml/v3_0_000/groups/a:b/members
     * the urlStrings would be size three: {"group", "a:b", "member"}
     * @param sourceId in url (if applicable)
     * @param subjectId in url (if applicable)
     * @param requestObject is the request body converted to object
     * @return the return object
     */
    @Override
    public WsResponseBean service(
        GrouperWsVersion clientVersion, String subjectId,  String sourceId, 
        List<String> urlStrings,
        WsRequestBean requestObject) {

      if (GrouperUtil.length(urlStrings) == 0 && (requestObject == null || 
          requestObject instanceof WsRestGetGroupsLiteRequest)) {

        WsRestGetGroupsLiteRequest wsRestGetGroupsLiteRequest = 
          (WsRestGetGroupsLiteRequest)requestObject;
        
        //url should be: /xhtml/v1_3_000/subjects/12345/groups
        return GrouperServiceRest.getGroupsLite(clientVersion, subjectId, 
            sourceId, wsRestGetGroupsLiteRequest);
        
      }
      
      throw new RuntimeException("Invalid REST GET Subject / Group request: " + clientVersion 
          + " , subjectId: " + subjectId + ", sourceId: " + sourceId  
          + ", " + GrouperUtil.toStringForLog(urlStrings) + ", requestObject: " 
          + GrouperUtil.className(requestObject));
    }

  };

  /**
     * handle the incoming request based on HTTP method GET and group as resource
     * @param clientVersion version of client, e.g. v1_3_000
     * @param urlStrings not including the app name or servlet.  
     * for http://localhost/grouper-ws/servicesRest/xhtml/v3_0_000/groups/a:b/members
     * the urlStrings would be size three: {"group", "a:b", "members"}
     * @param sourceId in url (if applicable)
     * @param subjectId in url (if applicable)
     * @param requestObject is the request body converted to object
     * @return the return object
   */
  public abstract WsResponseBean service(
      GrouperWsVersion clientVersion, String subjectId, String sourceId, 
      List<String> urlStrings,
      WsRequestBean requestObject);

  /**
   * do a case-insensitive matching
   * 
   * @param string
   * @param exceptionOnNotFound true if exception should be thrown on not found
   * @return the enum or null or exception if not found
   * @throws GrouperRestInvalidRequest if there is a problem
   */
  public static GrouperWsRestGetSubject valueOfIgnoreCase(String string,
      boolean exceptionOnNotFound) throws GrouperRestInvalidRequest {
    return GrouperServiceUtils.enumValueOfIgnoreCase(GrouperWsRestGetSubject.class, 
        string, exceptionOnNotFound);
  }

}
